package com.jiuqi.sql.impl;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.jiuqi.sql.DBException;
import com.jiuqi.sql.FieldInfo;
import com.jiuqi.sql.FieldInfoList;
import com.jiuqi.sql.InputDataSet;
import com.jiuqi.sql.InputDataSetMetaData;
import com.jiuqi.sql.InputRecordBuffer;
import com.jiuqi.sql.impl.DBaseFieldHeader;
import com.jiuqi.sql.impl.DBaseFileHeader;
import com.jiuqi.sql.impl.DBaseFileTail;
import com.jiuqi.sql.impl.DBaseInputDataSetMetaData;
import com.jiuqi.sql.impl.DBaseInputRecordBuffer;
import com.jiuqi.util.MemStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

public class DBaseInputDataSet extends InputDataSet {
  private InputStream stream;
  
  private DBaseFileHeader header;
  
  private DBaseFileTail tail;
  
  private DBaseInputRecordBuffer buffer;
  
  private FieldInfoList infoList;
  
  private int getFieldType(char dataType) {
    int result = dataType;
    switch (dataType) {
      case 'C':
      case 'F':
      case 'W':
        result = 6;
        break;
      case 'L':
        result = 1;
        break;
      case '+':
        result = 7;
        break;
      case '%':
      case 'O':
        result = 3;
        break;
      case 'A':
      case 'I':
      case 'R':
        result = 5;
        break;
      case '@':
      case 'D':
        result = 2;
        break;
      case 'Z':
        result = 8;
        break;
    } 
    return result;
  }
  
  public DBaseInputDataSet(InputStream inStream) {
    this.header = new DBaseFileHeader();
    this.tail = new DBaseFileTail();
    this.stream = inStream;
  }
  
  public DBaseInputDataSet(byte[] data) {
    this(new ByteArrayInputStream(data));
  }
  
  public FieldInfoList getFields() {
    return this.infoList;
  }
  
  public InputDataSetMetaData getMetaData() throws DBException {
    return (InputDataSetMetaData)new DBaseInputDataSetMetaData(this.infoList);
  }
  
  protected boolean getNextRecord(InputRecordBuffer buffer) throws DBException {
    return buffer.readRecord(this.stream);
  }
  
  protected void initFields(FieldInfoList fieldInfos) throws DBException {
    fieldInfos.clear();
    try {
      this.header.readFromStream(this.stream);
      boolean supportText = (this.header.ver == -115);
      int fieldCount = (this.header.structDefLen - 68 - 1) / 50;
      if (fieldCount == 0)
        throw new DBException("Not Field Define！"); 
      DBaseFieldHeader fieldHeader = new DBaseFieldHeader();
      int fieldOffset = 0;
      for (int i = 0; i < fieldCount; i++) {
        fieldHeader.readFromStream(this.stream);
        FieldInfo fieldInfo = fieldInfos.add();
        fieldInfo.name = fieldHeader.fieldName;
        fieldInfo.incValue = fieldHeader.incValue;
        fieldInfo.offset = fieldOffset;
        fieldInfo.size = fieldHeader.fieldLength;
        fieldInfo.decimal = fieldHeader.scale;
        fieldInfo.type = getFieldType(fieldHeader.dataType);
        if (supportText && fieldInfo.type == 6) {
          fieldOffset += 4;
        } else {
          fieldOffset += fieldHeader.fieldLength;
        } 
      } 
      if (fieldOffset != this.header.recLength - 1)
        throw new DBException("Field Define Infomation Error"); 
      this.stream.read();
      this.stream.mark(0);
      this.stream.skip(this.header.recCount * this.header.recLength + 1L);
      this.tail.readFromStream(this.stream);
      this.buffer = new DBaseInputRecordBuffer(this.header.recLength - 1, supportText, this.tail);
      this.infoList = fieldInfos;
      this.stream.reset();
    } catch (IOException ex) {
      DBException dbe = new DBException();
      dbe.setStackTrace(ex.getStackTrace());
      throw dbe;
    } 
  }
  
  protected InputRecordBuffer internalOpen() throws DBException {
    return (InputRecordBuffer)this.buffer;
  }
  
  protected void internalClose() throws DBException {
    this.buffer = null;
  }
  
  public void first() throws DBException {
    try {
      this.stream.reset();
    } catch (Exception ex) {
      throw new DBException(ex);
    } 
  }
  
  public static void main(String[] args) throws Exception {
    MemStream memStream = new MemStream();
    memStream.loadFromFile("d:\\1.dbf");
    InputStream as = new ByteArrayInputStream(memStream.getBytes());
    InputDataSet dataset = new com.jiuqi.sql.impl.DBaseInputDataSet(as);
    dataset.open();
    while (dataset.next()) {
      System.out.print(dataset.getBoolean(1));
      if (dataset.wasNull())
        System.out.print("[ISNULL]"); 
      System.out.print("\t");
      System.out.print(dataset.getDouble(2));
      if (dataset.wasNull())
        System.out.print("[ISNULL]"); 
      System.out.print("\t");
      System.out.print(dataset.getInt(3));
      if (dataset.wasNull())
        System.out.print("[ISNULL]"); 
      System.out.print("\t");
      System.out.print(dataset.getString(4));
      if (dataset.wasNull())
        System.out.print("[ISNULL]"); 
      System.out.print("\t");
      System.out.print(dataset.getLong(5));
      if (dataset.wasNull())
        System.out.print("[ISNULL]"); 
      System.out.println("\t");
    } 
    System.out.println("OK");
  }
}
